# coding=utf-8

from .projects import projects_commands
from .auth import auth_commands
from .orgs import orgs_commands
from .experiments import experiments_commands
from .run_code import runcode_commands



# coding=utf-8
import os

try:
    import ConfigParser as configparser
except ImportError:
    import configparser

_missing_link_config = 'missinglink.cfg'


default_api_host = 'https://missinglinkai.appspot.com'
default_host = 'https://missinglink.ai'
default_client_id = 'nbkyPAMoxj5tNzpP07vyrrsVZnhKYhMj'
default_auth0 = 'missinglink'


def find_first_file(filename):
    possible_paths = [os.getcwd(), os.path.expanduser('~')]

    for possible_path in possible_paths:
        candidate_config_file = os.path.join(possible_path, filename)

        if os.path.isfile(candidate_config_file):
            return candidate_config_file

    return filename


class Config(object):
    def __init__(self, config_prefix):
        self.config_prefix = config_prefix

        config_file_abs_path = find_first_file(_missing_link_config)

        parser = configparser.RawConfigParser()
        parser.read([config_file_abs_path])

        self.parser = parser

        readonly_parser = configparser.RawConfigParser()
        readonly_parser.read([self.prefixed_config_name])

        self.readonly_parser = readonly_parser

    @property
    def prefixed_config_name(self):
        return '%s-%s' % (self.config_prefix, _missing_link_config)

    @property
    def api_host(self):
        return self.general_config.get('api_host', default_api_host)

    @property
    def host(self):
        return self.general_config.get('host', default_host)

    @property
    def client_id(self):
        return self.general_config.get('client_id', default_client_id)

    @property
    def auth0(self):
        return self.general_config.get('auth0', default_auth0)

    def get_prefix_section(self, section):
        return '%s-%s' % (self.config_prefix, section) if self.config_prefix else section

    @property
    def token_config(self):
        section_name = self.get_prefix_section('token')
        return self.items(section_name, most_exists=False)

    @property
    def general_config(self):
        return self.readonly_items('general', most_exists=self.config_prefix is not None)

    @property
    def refresh_token(self):
        return self.token_config.get('refresh_token')

    @property
    def id_token(self):
        return self.token_config.get('id_token')

    @classmethod
    def items_from_parse(cls, parser, section, most_exists):
        try:
            return dict(parser.items(section))
        except configparser.NoSectionError:
            if most_exists:
                raise

            return {}

    def items(self, section, most_exists=False):
        return self.items_from_parse(self.parser, section, most_exists=most_exists)

    def readonly_items(self, section, most_exists=False):
        return self.items_from_parse(self.readonly_parser, section, most_exists=most_exists)

    def set(self, section, key, val):
        try:
            self.parser.add_section(section)
        except configparser.DuplicateSectionError:
            pass

        self.parser.set(section, key, val)

    def _write(self, fo):
        self.parser.write(fo)

    def save(self):
        config_file_abs_path = find_first_file(_missing_link_config)

        with open(config_file_abs_path, 'w') as configfile:
            self._write(configfile)

    def update_and_save(self, d):
        for section, section_values in d.items():
            section_name_with_prefix = self.get_prefix_section(section)
            for key, val in section_values.items():
                self.set(section_name_with_prefix, key, val)

        self.save()



# coding=utf-8
import requests
import fudge
import json

from fudge.inspector import arg

from click.testing import CliRunner

from tests.base import BaseCliTest
from mali import cli


class TestExperiments(BaseCliTest):
    project_id = BaseCliTest.some_random_shit_number_int63()
    experiment_id = BaseCliTest.some_random_shit_number_int63()
    user_sent_metrics = {
        BaseCliTest.some_random_shit(): BaseCliTest.some_random_shit()
    }

    @fudge.patch('mali_commands.commons.handle_api')
    def testListExperiments(self, handle_api_mock):
        handle_api_mock.expects_call().with_matching_args(
            arg.any(),  # ctx
            requests.get,
            'projects/{project_id}/experiments'.format(project_id=self.project_id)
        ).returns({
            'experiments': [
                {
                    'experiment_id': BaseCliTest.some_random_shit_number_int63(),
                    'created_at': '2017-05-04T07:18:47.620155',
                    'display_name': BaseCliTest.some_random_shit(),
                    'description': BaseCliTest.some_random_shit()
                }
            ]
        })

        runner = CliRunner()
        result = runner.invoke(cli, ['experiments', 'list', '--projectId', self.project_id], catch_exceptions=False)
        self.assertEqual(result.exit_code, 0)

    @fudge.patch('mali_commands.commons.handle_api')
    def testUpdateExperimentMetrics_withValidJSON_expectsHandleApiCallAndSuccessfulExit(self, handle_api_mock):
        handle_api_mock.expects_call().with_matching_args(
            arg.any(),  # ctx
            requests.post,
            'projects/{project_id}/experiments/{experiment_id}/metrics'.format(project_id=self.project_id,
                                                                               experiment_id=self.experiment_id),
            self.user_sent_metrics
        ).returns({})

        runner = CliRunner()
        result = runner.invoke(cli, ['experiments', 'update_experiment_metrics', '--projectId', self.project_id,
                                     '--experimentId', self.experiment_id, '--metrics',
                                     json.dumps(self.user_sent_metrics)], catch_exceptions=False)
        self.assertEqual(result.exit_code, 0)

    @fudge.patch('mali_commands.commons.handle_api')
    def testUpdateExperimentMetrics_withValidJSONAndShortHands_expectsHandleApiCallAndSuccessfulExit(self, handle_api_mock):
        handle_api_mock.expects_call().with_matching_args(
            arg.any(),  # ctx
            requests.post,
            'projects/{project_id}/experiments/{experiment_id}/metrics'.format(project_id=self.project_id,
                                                                               experiment_id=self.experiment_id),
            self.user_sent_metrics
        ).returns({})

        runner = CliRunner()
        result = runner.invoke(cli, ['experiments', 'update_experiment_metrics', '-p', self.project_id,
                                     '-e', self.experiment_id, '-m', json.dumps(self.user_sent_metrics)],
                               catch_exceptions=False)
        self.assertEqual(result.exit_code, 0)

    def testUpdateExperimentMetrics_withInvalidJSON_expectsRaisingOfValueError(self):
        self.assertRaises(ValueError, CliRunner().invoke, cli, ['experiments', 'update_experiment_metrics',
                                                                '--projectId', self.project_id, '--experimentId',
                                                                self.experiment_id, '--metrics',
                                                                BaseCliTest.some_random_shit()],
                          catch_exceptions=False)

    @fudge.patch('mali_commands.commons.handle_api')
    def testUpdateModelMetrics_withValidJSON_expectsHandleApiCallAndSuccessfulExit(self, handle_api_mock):
        model_weights_hash = BaseCliTest.some_random_shit()
        handle_api_mock.expects_call().with_matching_args(
            arg.any(),  # ctx
            requests.post,
            'model_weights_hashes/{model_weights_hash}/metrics'.format(model_weights_hash=model_weights_hash),
            self.user_sent_metrics
        ).returns({})

        runner = CliRunner()
        result = runner.invoke(cli, ['experiments', 'update_model_metrics', '--weightsHash', model_weights_hash,
                                     '--metrics', json.dumps(self.user_sent_metrics)], catch_exceptions=False)
        self.assertEqual(result.exit_code, 0)



# coding=utf-8
import fudge
import httpretty
from fudge.inspector import arg

from mali import cli
from .base import BaseCliTest
from click.testing import CliRunner


class TestAuth(BaseCliTest):
    client_id = 'client_id_' + BaseCliTest.some_random_shit()
    auth0 = 'auth0_auth_' + BaseCliTest.some_random_shit()
    access_token = 'access_token_' + BaseCliTest.some_random_shit()
    refresh_token = 'refresh_token_' + BaseCliTest.some_random_shit()
    id_token = 'id_token_' + BaseCliTest.some_random_shit()

    @httpretty.activate
    def test_whoami(self):
        runner = CliRunner()
        result = runner.invoke(cli, ["auth", "whoami"], catch_exceptions=False)
        self.assertEquals(result.exit_code, 0)

    @httpretty.activate
    @fudge.patch('mali_commands.commons.pixy_flow')
    @fudge.patch('mali_commands.config.Config.update_and_save')
    def test_init_auth(self, pixy_flow, update_and_save_config):
        access_token = 'access_token'
        refresh_token = 'refresh_token'
        id_token = 'id_token'

        pixy_flow.expects_call().returns((access_token, refresh_token, id_token)).times_called(1)
        update_and_save_config.expects_call().with_args(
            {
                'token': {
                    'access_token': access_token,
                    'id_token': id_token,
                    'refresh_token': refresh_token
                }
            }
        ).returns(None).times_called(1)

        runner = CliRunner()
        result = runner.invoke(cli, ["auth", "init"], catch_exceptions=False)
        self.assertEquals(result.exit_code, 0)

    @httpretty.activate
    @fudge.patch('werkzeug.serving.make_server')
    @fudge.patch('webbrowser.open')
    @fudge.patch('mali_commands.commons.auth0_oauth_token')
    def test_pixy_flow(self, make_server, webbrowser_open, auth0_oauth_token):
        from mali_commands.commons import pixy_flow, build_auth0_url

        port = self.some_random_shit_number_int63()
        srv = fudge.Fake().has_attr(port=port).provides('serve_forever')

        make_server.expects_call().with_args('127.0.0.1', arg.any(), arg.any()).returns(srv)

        ctx = fudge.Fake().has_attr(local_web_server=True, client_id=self.client_id, auth0=self.auth0)

        webbrowser_open.expects_call()

        verifier = arg.any()
        code = None

        redirect_url = 'http://localhost:%s/admin/auth/init' % port

        auth0_oauth_token.expects_call()\
            .with_args(build_auth0_url(self.auth0), code, verifier, self.client_id, redirect_url).returns({
                'access_token': self.access_token,
                'refresh_token': self.refresh_token,
                'id_token': self.id_token,
            })

        pixy_flow(ctx)



import click
import requests
from .commons import add_to_data_if_not_none, output_result


@click.group('code')
def runcode_commands():
    pass


@runcode_commands.command('create')
@click.pass_context
@click.option('--name', required=True)
@click.option('--git', required=True)
@click.option('--org', required=False)
def code_crate(ctx, name, git, org):
    data = {}

    add_to_data_if_not_none(data, name, "name")
    add_to_data_if_not_none(data, git, "git")
    add_to_data_if_not_none(data, org, "org")

    result = ctx.obj.handle_api(ctx.obj, requests.post, 'runcode', data)

    output_result(ctx, result, ['ok'])



# coding=utf-8
import click
import requests
import json

from mali_commands.commons import output_result
from mali_commands.options import project_id_option, experiment_id_option, metrics_option, model_weights_hash_option
from mali_commands.utilities.formatters import format_datetime, truncate_long_text, truncate_short_text


@click.group('experiments')
def experiments_commands():
    pass


@experiments_commands.command('list')
@project_id_option(required=True)
@click.pass_context
def list_experiments(ctx, projectid):
    """List experiments of a project.
    """
    list_experiments_path = 'projects/{project_id}/experiments'.format(project_id=projectid)
    result = ctx.obj.handle_api(ctx.obj, requests.get, list_experiments_path)
    experiments = result.get('experiments', [])

    displayed_fields = ['experiment_id', 'created_at', 'display_name', 'description']
    formatters = {
        'created_at': format_datetime,
        'display_name': truncate_short_text,
        'description': truncate_long_text,
    }
    output_result(ctx, experiments, displayed_fields, formatters=formatters)


@experiments_commands.command('update_experiment_metrics')
@project_id_option(required=True)
@experiment_id_option(required=True)
@metrics_option(required=True)
@click.pass_context
def update_experiment_metrics(ctx, projectid, experimentid, metrics):
    """Send experiment metrics to an experiment.

    Example:

    To send metrics to the 5rd experiment of the project "123", run

    \b
        mali experiments update_experiment_metrics --projectId 123 --experimentId 5 --metrics '{"ex_cost": 99}'
    """
    update_metrics_path = 'projects/{project_id}/experiments/{experiment_id}/metrics'\
        .format(project_id=projectid, experiment_id=experimentid)
    data = _get_metrics_json(metrics)

    result = ctx.obj.handle_api(ctx.obj, requests.post, update_metrics_path, data)
    output_result(ctx, result, ['ok'])


@experiments_commands.command('update_model_metrics')
@model_weights_hash_option(required=True)
@metrics_option(required=True)
@click.pass_context
def update_model_metrics(ctx, weightshash, metrics):
    """Send the model's metrics to its corresponding experiment epochs.

    Each model weights hash is attached to certain experiment epochs and thus can be used to send
    metrics that are relevant to those epochs. The model weights hash is a hexadecimal string. To
    calculate the weights hash of a model:

        - Calculate the sha1 strings of the weights for each layers

        - Calculate the sha1 string of the combine hashes. For example, the model has 3 layers with
    the layers' weight hashes: ['abc', '123', 'def'], the model weight hash is `sha1('abc123def')`

    Alternatively, you can use `missinglink-sdk` Python package to calculate the weights hash for
    your model. Each framework callback has its corresponding `get_weights_hash` method.

    WARNING: The same model weights hash can appear in different experiments or different epochs
    (for example, when running the same net twice). As such, this command will send the metrics to
    all these experiments/epochs that it can identify from the hash.

    Example:

    To send metrics of the model which hash is 324e16b5e, run

    \b
        mali experiments update_model_metrics --weightsHash 324e16b5e --metrics '{"ex_cost": 99}'

    """
    update_model_metrics_path = 'model_weights_hashes/{model_weights_hash}/metrics'\
        .format(model_weights_hash=weightshash)
    data = _get_metrics_json(metrics)

    result = ctx.obj.handle_api(ctx.obj, requests.post, update_model_metrics_path, data)
    output_result(ctx, result, ['ok'])


def _get_metrics_json(metrics_string):
    try:
        return json.loads(metrics_string)
    except ValueError:
        raise ValueError('The metrics supplied: "{metrics_supplied}" is not a valid JSON dictionary format.'
                         .format(metrics_supplied=metrics_string))
